import {attribute, DataStore, Model, model} from '@rx-suite/data';
import * as yup from 'yup';
import {t} from "@lingui/macro";
import {computed} from "mobx";
import {ComponentPermissionModel, ComponentPermissionStore} from "./ComponentPermissionStore";
import {RolePermissionModel, RolePermissionStore} from "./RolePermissionStore";
import {moduleEnabled, moduleFeatureEnabled} from "@rx-suite/stores";
import {RxModuleComponentStore} from "./RxModuleComponentStore";
import {ComponentPermissionGroup} from "../models/ComponentPermissionGroup";

export type AppsPermissionGroup = {
    id: string
    module_name?: string
    components: Array<ComponentPermissionGroup>
}

@model({
    transportLayer: "/api/roles",
    permissionTarget: "Role",
    routingInfo: {
        create: '/roles/create',
        read: '/roles/:id/read',
        update: '/roles/:id/edit',
        list: '/roles'
    }
})

export class RoleModel extends Model {
    @attribute({
        label: t`Name`,
        description: t`The name that will be used to identify this role.`,
        validation: yup.string().required(t`Name is a required field`)
    })
    name = '';

    constructor() {
        super();
        super.setup();
    }

    get asJson(): Record<string, unknown> {
        return {
            ...this.toJSON(),
            role_permissions_attributes: this.permissions.map(rp => {
                rp.rolePermission.viewModel.submit();
                return rp.rolePermission.asJson
            })
        }
    }

    @computed
    get enabledPermissionsGroupedByApp() {
        const apps: { [key: string]: Permission[] } = {}
        this.enabledPermissions.forEach(p => {
            const id = p.componentPermission.component_id.toString()
            apps[id] = apps[id] || [];
            apps[id].push(p);
        });
        return apps;
    }

    @computed
    get enabledPermissions() {
        return this.permissions.filter(permission => {
            if (permission.componentPermission.feature_module_id) {
                if (permission.componentPermission.feature_subfeature_id) {
                    return moduleFeatureEnabled(permission.componentPermission.feature_module_id, permission.componentPermission.feature_subfeature_id);
                } else {
                    return moduleEnabled(permission.componentPermission.feature_module_id);
                }
            }
            return true;
        });
    }

    @computed
    get permissionsByApp(): Array<AppsPermissionGroup> {
        return Object.keys(this.enabledPermissionsGroupedByApp).map(key => {
            const permissions = this.enabledPermissionsGroupedByApp[key];
            const components = permissions.reduce((acc: Array<any>, permission) => {
                let index = acc.map(component => component.target.target_id.toString()).indexOf(permission.componentPermission.target_id.toString());

                // component does not exists, create it
                if (index === -1) {
                    const cpg = new ComponentPermissionGroup({
                        target_id: permission.componentPermission.target_id,
                        target_display_name: permission.componentPermission.target_display_name,
                        target_category: permission.componentPermission.target_category
                    });

                    acc.push(cpg)

                    index = acc.length - 1;
                }

                // add a new action
                acc[index].actions.push({
                    componentPermission: permission.componentPermission,
                    rolePermission: permission.rolePermission
                });

                return acc;
            }, []);

            // Through the magic of caching, this isn't a network request (probably)
            const app = RxModuleComponentStore.get(key)

            return {
                id: key,
                module_name: app?.module_name as string,
                components: components
            }
        });
    }

    // Returns an array of all the component permissions and the associated role permissions
    @computed get permissions() {
        const roleRP = (RolePermissionStore.models || []).filter(rp => rp.role_id === this.id) as RolePermissionModel[];
        const componentPermissions = (ComponentPermissionStore.models || []) as ComponentPermissionModel[];
        //ensure every component permission has an matching role permission
        return componentPermissions.reduce((acc: Array<Permission>, cp: ComponentPermissionModel) => {
            let rp = roleRP.find((rp: RolePermissionModel) => cp.id === rp.component_permission_id);

            if (!rp) {
                rp = RolePermissionStore.createModel({
                    "component_permission_id": cp.id,
                    "permitted": false
                } as RolePermissionModel)
            }

            acc.push({componentPermission: cp, rolePermission: rp});

            return acc;
        }, []);
    }
}

export type Permission = {
    componentPermission: ComponentPermissionModel
    rolePermission: RolePermissionModel
}

export const RoleStore = RoleModel.store as unknown as DataStore<RoleModel>;
